pub struct ReadySubscription { /* private fields */ }Expand description
A subscription to the Ready state, typically held by futures
that require the ready signal.
Implementations
Sourceimpl ReadySubscription
impl ReadySubscription
Sourcepub async fn wait(self)
pub async fn wait(self)
Asynchronously wait for the ready signal under SSR.
This may contain a receiver that will wait for the signal from
the associated Ready which this subscription belongs to. If
no such receiver is in fact available (due to how the associated
handle providing this subscription was set
up), or that a ready signal was already broadcasted, this
will return immediately, otherwise it will wait for the ready
message to arrive until execution will be allowed to continue.
Under CSR this is essentially a no-op.